#!/usr/bin/env bash

set -e

# Initialize variables
CLUSTER_NAME=""
RES_GROUP_NAME=""
CLUSTER_LOCATION=""
SUBSCRIPTION_ID=""
TENANT_ID=""
CLIENT_ID=""
AZURE_MANAGED_IDENTITY=""
AZURE_CLIENT_SECRET=""
HELM_VALUES=""

RELEASE_NAME="elotl-luna"          # Default, can be overridden by --helm-release-name
CLUSTER_NAMESPACE="elotl"          # Default, can be overridden by --namespace
LUNA_SERVICE_ACCOUNT_PREFIX="luna" # Default, can be overridden by --service-account-prefix

CHART_PATH=${CHART_PATH:-"./elotl-luna-v*.tgz"}

cat <<EOF
Thank you for deploying and testing Luna!
Your participation is invaluable to us. If you have any feedback, questions,
or simply want to engage further, please don’t hesitate to join our
Slack channel:
https://join.slack.com/t/elotl-free-trial/shared_invite/zt-1tciz8cck-H9Swzl2grCqPaLJeHYtbBQ

Or email us at info@elotl.co.

Your input helps us improve and refine our product.
EOF

# Function to display an error message and exit
display_error() {
    echo "Error: $1"
    exit 1
}

# Function to display the script usage
display_usage() {
  echo "Usage: $0 --name <cluster-name> --resource-group <resource-group-name> --location <cluster-location> --subscription <subscription-id> --tenant <tenant-id> --id <client-id> (--identity-name <managed-identity> or --client-secret <client-secret>) [--helm-release-name <release-name>] [--namespace <namespace>] [--service-account-prefix <prefix>] [--additional-helm-values \"<additional-helm-values>\"]"
    exit 1
}

# Function to parse command line arguments
parse_arguments() {
    while [[ $# -gt 0 ]]; do
        key="$1"
        case $key in
            --name)
                shift
                if [[ -z $1 ]]; then
                    display_error "Cluster name not provided for --name option."
                fi
                CLUSTER_NAME="$1"
                ;;
            --resource-group)
                shift
                if [[ -z $1 ]]; then
                    display_error "Resource group name not provided for --resource-group option."
                fi
                RES_GROUP_NAME="$1"
                ;;
            --location)
                shift
                if [[ -z $1 ]]; then
                    display_error "Cluster location not provided for --location option."
                fi

                # Check if the given location is valid
                available_locations=$(az account list-locations --query '[].name' -o tsv)
                if ! grep -q "^$1$" <<< "$available_locations"; then
                  display_error "Cluster location provided ($1) for --location option is not valid."
                fi

                CLUSTER_LOCATION="$1"
                ;;
            --subscription)
                shift
                if [[ -z $1 ]]; then
                    display_error "Subscription ID not provided for --subscription option."
                fi
                SUBSCRIPTION_ID="$1"
                ;;
            --tenant)
                shift
                if [[ -z $1 ]]; then
                    display_error "Tenant ID not provided for --tenant option."
                fi
                TENANT_ID="$1"
                ;;
            --id)
                shift
                if [[ -z $1 ]]; then
                    display_error "Client ID not provided for --id option."
                fi
                CLIENT_ID="$1"
                ;;
            --identity-name)
                shift
                if [[ $AZURE_CLIENT_SECRET != "" ]]; then
                    display_error "Both --identity-name and --client-secret options cannot be used together."
                fi
                if [[ -z $1 ]]; then
                    display_error "Managed identity not provided for --identity-name option."
                fi
                AZURE_MANAGED_IDENTITY="$1"
                ;;
            --client-secret)
                shift
                if [[ $AZURE_MANAGED_IDENTITY != "" ]]; then
                    display_error "Both --identity-name and --client-secret options cannot be used together."
                fi
                if [[ -z $1 ]]; then
                    display_error "Client secret not provided for --client-secret option."
                fi
                AZURE_CLIENT_SECRET="$1"
                ;;
            --helm-release-name)
                shift
                if [[ -z $1 ]]; then
                    display_error "Release name not provided for --helm-release-name option."
                fi
                RELEASE_NAME="$1"
                ;;
            --namespace)
                shift
                if [[ -z $1 ]]; then
                    display_error "Namespace not provided for --namespace option."
                fi
                CLUSTER_NAMESPACE="$1"
                ;;
            --service-account-prefix)
                shift
                if [[ -z $1 ]]; then
                    display_error "Service Account Prefix not provided for --service-account-prefix option."
                fi
                LUNA_SERVICE_ACCOUNT_PREFIX="$1"
                ;;
            --additional-helm-values)
                shift
                if [[ -z $1 ]]; then
                    display_error "Additional Helm values not provided for --additional-helm-values option."
                fi
                HELM_VALUES="$1"
                ;;
            *)
                display_error "Unrecognized option: $key"
                ;;
        esac
        shift
    done
}

# Function to validate the required options
validate_options() {
    if [[ (-z $CLUSTER_NAME || -z $RES_GROUP_NAME || -z $CLUSTER_LOCATION || -z $SUBSCRIPTION_ID || -z $TENANT_ID || -z $CLIENT_ID || -z $RELEASE_NAME || -z $CLUSTER_NAMESPACE) || (-z $AZURE_MANAGED_IDENTITY && -z $AZURE_CLIENT_SECRET) || (-n $AZURE_MANAGED_IDENTITY && -n $AZURE_CLIENT_SECRET) ]]; then
        display_usage
    fi
}

# Function to display the values of the variables
display_values() {
    echo "Cluster Name:           $CLUSTER_NAME"
    echo "Resource Group Name:    $RES_GROUP_NAME"
    echo "Cluster Location:       $CLUSTER_LOCATION"
    echo "Subscription ID:        $SUBSCRIPTION_ID"
    echo "Tenant ID:              $TENANT_ID"
    echo "Client ID:              $CLIENT_ID"
    echo "Managed Identity:       $AZURE_MANAGED_IDENTITY"
    echo "Client Secret:          $AZURE_CLIENT_SECRET"
    echo "Helm Release Name:      $RELEASE_NAME"
    echo "Namespace:              $CLUSTER_NAMESPACE"
    echo "Additional Helm Values: $HELM_VALUES"
    echo "Service Account Prefix: $LUNA_SERVICE_ACCOUNT_PREFIX"
}

# Function to prompt the user to verify the values
prompt_verification() {
    if [[ -t 1 ]]; then
        echo "Please verify the values displayed above."
        while true; do
            read -rp "Are the values correct? (Y/N): " choice
            case $choice in
                [Yy])
                    break
                    ;;
                [Nn])
                    echo "Script execution cancelled by the user."
                    exit 1
                    ;;
                *)
                    echo "Invalid input. Please enter Y or N."
                    ;;
            esac
        done
    fi
}

extract_helm_value() {
    local pattern="$1"

    # Remove leading and trailing whitespace
    input_string=$(echo "$HELM_VALUES" | xargs)

    # Find the substring that matches the pattern
    substring=$(echo "$input_string" | grep -oE -- "$pattern=[^[:space:]]+")

    if [ -n "$substring" ]; then
        # Extract the value
        value=$(echo "$substring" | cut -d= -f2)
        echo "$value"
    fi
}

# Check if OIDC issuer is enabled for the cluster
check_oidc_issuer() {
    # Check if OIDC issuer is enabled

    cluster_info=$(az aks show --name $CLUSTER_NAME --resource-group $RES_GROUP_NAME --query "oidcIssuerProfile.enabled")

    echo "Checking if OIDC issuer is Enable for Cluster $CLUSTER_NAME"
    if [[ $cluster_info = "true" ]]; then
        echo "OIDC issuer is enabled for the cluster."
        return 0
    else
        display_error "OIDC issuer is not enabled for the cluster."
    fi
}

# Check if Workload identity is enabled for the cluster
check_workload_identity() {
    # Check if workload identity is enabled

    cluster_info=$(az aks show --name $CLUSTER_NAME --resource-group $RES_GROUP_NAME --query "identity")

    echo "Checking if Workload Identity is Enable for Cluster $CLUSTER_NAME"
    if [[ -n $cluster_info ]]; then
        echo "Workload identity is enabled for the cluster."
        return 0
    else
        display_error "Workload identity is not enabled for the cluster."
    fi
}

no_helm_chart_installed() {
    if helm -n ${CLUSTER_NAMESPACE} status ${RELEASE_NAME} >/dev/null 2>&1
    then
        cat <<EOF
Upgrading Luna from this script isn’t supported. Please run the following w/<retained-values-file> set to <retained-path>/${CLUSTER_NAME}_values_full.yaml, for post Luna 0.5.4 installation, or <retained-path>/${CLUSTER_NAME}_values.yaml otherwise:

    $ helm upgrade ${RELEASE_NAME} $CHART_PATH --wait --namespace=${CLUSTER_NAMESPACE} --values=<retained-values-file> $HELM_VALUES
EOF
        exit 1
    fi
}

deploy_cert_manager_if_needed() {
  if kubectl wait pods -A -l app.kubernetes.io/instance=cert-manager --for=condition=ready &>/dev/null ; then
    echo "detected cert-manager running"
  else
    echo "no cert-manager deployment found, installing a new one"
    kubectl apply -f https://github.com/cert-manager/cert-manager/releases/download/v1.11.1/cert-manager.yaml &>/dev/null
    if kubectl wait pods -A -l app.kubernetes.io/instance=cert-manager --for=condition=ready --timeout=5m &>/dev/null ; then
      echo "successfully installed cert-manager"
    else
      echo "unable to install cert-manager, aborting"
      exit 1
    fi
  fi
}

deploy() {
    echo "deploying changes to namespace ${CLUSTER_NAMESPACE} in ${RES_GROUP_NAME} AKS resource group"
    if [ -n "$AZURE_MANAGED_IDENTITY" ] ; then
      echo "Creating federated-credential for azure managed identity w/propagation delay..."
      AZURE_OIDC_ISSUER="$(az aks show -n ${CLUSTER_NAME} -g "${RES_GROUP_NAME}" --query "oidcIssuerProfile.issuerUrl" -otsv)"
      az identity federated-credential create --name luna-manager-federated-identity --identity-name "${AZURE_MANAGED_IDENTITY}" --resource-group "${RES_GROUP_NAME}" --issuer "${AZURE_OIDC_ISSUER}" --subject system:serviceaccount:"${CLUSTER_NAMESPACE}":"${LUNA_SERVICE_ACCOUNT_PREFIX}"-manager --audience api://AzureADTokenExchange
      sleep 10
    fi
    echo "Creating resources..."
    ./create_resources.sh "$RES_GROUP_NAME" "$CLUSTER_NAME" "$CLUSTER_NAMESPACE" "$CLUSTER_LOCATION" "$SUBSCRIPTION_ID" "$TENANT_ID" "$CLIENT_ID" "$AZURE_MANAGED_IDENTITY" "$LUNA_SERVICE_ACCOUNT_PREFIX"
    echo "deploying luna-manager helm chart in namespace $CLUSTER_NAMESPACE"
    echo "setting scaleDown.skipNodesWithLocalStorage=false to avoid metrics server local storage use of EmptyDir from blocking scale down; you can override by explicitly setting in [helm_values]"
    helm install "$RELEASE_NAME" $CHART_PATH \
        --wait \
        --create-namespace \
        --namespace="${CLUSTER_NAMESPACE}" \
        --values="${CLUSTER_NAME}"_values.yaml \
        --set azure.clientSecret="${AZURE_CLIENT_SECRET}" \
        --set scaleDown.skipNodesWithLocalStorage=false \
        $HELM_VALUES
    HELM_fullnameOverride=$(extract_helm_value "fullnameOverride")
    FULL_RELEASE_NAME=""
    if [ ${RELEASE_NAME} = "elotl-luna" ] || [ "${HELM_fullnameOverride}" != "" ]
    then
      FULL_RELEASE_NAME=${RELEASE_NAME}
    else
      FULL_RELEASE_NAME=${RELEASE_NAME}-elotl-luna
    fi
    echo "Saving helm values to ${CLUSTER_NAME}_${FULL_RELEASE_NAME}_values_full.yaml"
    kubectl get cm ${FULL_RELEASE_NAME} -n ${CLUSTER_NAMESPACE} -o "jsonpath={.data['config\.yml']}" > ${CLUSTER_NAME}_${FULL_RELEASE_NAME}_values_full.yaml
}

parse_arguments "$@"
validate_options
display_values
prompt_verification

if [[ -n $AZURE_MANAGED_IDENTITY ]]; then
    check_oidc_issuer
    check_workload_identity
fi

no_helm_chart_installed
deploy_cert_manager_if_needed
deploy
